@@ -1,4 +1,4 @@
-#  -*- coding: utf-8 -*-
+#   -*- coding: utf-8 -*-
 # SPDX-License-Identifier: MPL-2.0
 # Copyright 2020-2021 John Mille <john@compose-x.io>
 
@@ -12,6 +12,7 @@ import requests
 import urllib3
 from compose_x_common.compose_x_common import keyisset, keypresent
 from troposphere import (
+    AWS_ACCOUNT_ID,
     AWS_NO_VALUE,
     AWS_PARTITION,
     AWS_REGION,
@@ -40,10 +41,10 @@ from troposphere.ecs import (
 )
 from troposphere.ecs import Service as EcsService
 from troposphere.ecs import SystemControl, TaskDefinition, Tmpfs, Ulimit, Volume
-from troposphere.iam import Policy
+from troposphere.iam import Policy, PolicyType
 from troposphere.logs import LogGroup
 
-from ecs_composex.common import FILE_PREFIX, LOG, NONALPHANUM
+from ecs_composex.common import FILE_PREFIX, LOG, NONALPHANUM, add_parameters
 from ecs_composex.common.cfn_params import ROOT_STACK_NAME, Parameter
 from ecs_composex.common.compose_volumes import (
     ComposeVolume,
@@ -65,7 +66,6 @@ from ecs_composex.ecs.docker_tools import (
     set_memory_to_mb,
 )
 from ecs_composex.ecs.ecs_conditions import USE_FARGATE_CON_T
-from ecs_composex.ecs.ecs_iam import add_service_roles
 from ecs_composex.ecs.ecs_params import (
     AWS_XRAY_IMAGE,
     EXEC_ROLE_T,
@@ -135,6 +135,7 @@ class ComposeService(object):
         self.env_files = []
         self.tmpfses = []
         self.ephemeral_storage = 0
+        self.x_ecs = set_else_none("x-ecs", self.definition, {})
         self.user = None
         self.group = None
         self.user_group = None
@@ -1222,14 +1223,6 @@ def add_policies(config, key, new_policies):
         )
         if name in existing_policy_names:
             return
-        if not keyisset("PolicyDocument", policy):
-            raise KeyError("You must set the policy document for the policy")
-        if (
-            keyisset("Version", policy["PolicyDocument"])
-            and not isinstance(policy["PolicyDocument"]["Version"], str)
-            or not keyisset("Version", policy["PolicyDocument"])
-        ):
-            policy["PolicyDocument"]["Version"] = "2012-10-17"
         policy_object = Policy(PolicyName=name, PolicyDocument=policy["PolicyDocument"])
         existing_policies.append(policy_object)
 
@@ -1253,7 +1246,6 @@ def identify_repo_credentials_secret(settings, task, secret_name):
     :param secret_name:
     :return:
     """
-    secret_arn = None
     for secret in settings.secrets:
         if secret.name == secret_name:
             secret_arn = secret.arn
@@ -1262,9 +1254,14 @@ def identify_repo_credentials_secret(settings, task, secret_name):
                     f"secret {secret_name} was not found in the defined secrets",
                     [s.name for s in settings.secrets],
                 )
-            if secret.kms_key_arn:
-                task.exec_role.Policies.append(
-                    Policy(
+            if (
+                secret.kms_key_arn
+                and task.template
+                and "RepositoryCredsKmsKeyAccess" not in task.template.resources
+            ):
+                task.template.add_resource(
+                    PolicyType(
+                        "RepositoryCredsKmsKeyAccess",
                         PolicyName="RepositoryCredsKmsKeyAccess",
                         PolicyDocument={
                             "Version": "2012-10-17",
@@ -1276,12 +1273,122 @@ def identify_repo_credentials_secret(settings, task, secret_name):
                                 }
                             ],
                         },
+                        Roles=[Ref(task.exec_role.name["ImportParameter"])],
                     )
                 )
             return secret_arn
     return None
 
 
+def set_ecs_cluster_logging_s3_access(settings, policy, role_stack):
+    if settings.ecs_cluster.log_bucket:
+        parameter = Parameter("EcsExecuteLoggingBucket", Type="String")
+        add_parameters(role_stack.stack_template, [parameter])
+        if isinstance(settings.ecs_cluster.log_bucket, FindInMap):
+            role_stack.Parameters.update(
+                {parameter.title: settings.ecs_cluster.log_bucket}
+            )
+        else:
+            role_stack.Parameters.update(
+                {parameter.title: Ref(settings.ecs_cluster.log_bucket.cfn_resource)}
+            )
+        policy.PolicyDocument["Statement"].append(
+            {
+                "Sid": "AllowDescribeS3Bucket",
+                "Action": ["s3:GetEncryptionConfiguration"],
+                "Resource": [
+                    Sub(f"arn:${{{AWS_PARTITION}}}:s3:::${{{parameter.title}}}")
+                ],
+                "Effect": "Allow",
+            }
+        )
+        policy.PolicyDocument["Statement"].append(
+            {
+                "Sid": "AllowS3BucketObjectWrite",
+                "Action": ["s3:PutObject"],
+                "Resource": [
+                    Sub(f"arn:${{{AWS_PARTITION}}}:s3:::${{{parameter.title}}}/*")
+                ],
+                "Effect": "Allow",
+            }
+        )
+
+
+def set_ecs_cluster_logging_kms_access(settings, policy, role_stack):
+    if settings.ecs_cluster.log_key:
+        parameter = Parameter("EcsExecuteLoggingEncryptionKey", Type="String")
+        add_parameters(role_stack.stack_template, [parameter])
+        if isinstance(settings.ecs_cluster.log_key, FindInMap):
+            role_stack.Parameters.update(
+                {parameter.title: settings.ecs_cluster.log_key}
+            )
+        else:
+            role_stack.Parameters.update(
+                {
+                    parameter.title: GetAtt(
+                        settings.ecs_cluster.log_key.cfn_resource, "Arn"
+                    )
+                }
+            )
+        policy.PolicyDocument["Statement"].append(
+            {
+                "Action": [
+                    "kms:Encrypt*",
+                    "kms:Decrypt*",
+                    "kms:ReEncrypt*",
+                    "kms:GenerateDataKey*",
+                    "kms:Describe*",
+                ],
+                "Resource": [Ref(parameter)],
+                "Effect": "Allow",
+            }
+        )
+
+
+def set_ecs_cluster_logging_cw_access(settings, policy, role_stack):
+    if settings.ecs_cluster.log_group:
+        parameter = Parameter("EcsExecuteLoggingGroup", Type="String")
+        add_parameters(role_stack.stack_template, [parameter])
+        if isinstance(settings.ecs_cluster.log_group, FindInMap):
+            role_stack.Parameters.update(
+                {parameter.title: settings.ecs_cluster.log_group}
+            )
+            arn_value = Sub(
+                f"arn:${{{AWS_PARTITION}}}:logs:${{{AWS_REGION}}}:"
+                f"${{{AWS_ACCOUNT_ID}}}:${{{parameter.title}}}:*"
+            )
+        else:
+            role_stack.Parameters.update(
+                {parameter.title: GetAtt(settings.ecs_cluster.log_group, "Arn")}
+            )
+            arn_value = Ref(parameter)
+        policy.PolicyDocument["Statement"].append(
+            {
+                "Sid": "AllowDescribingAllCWLogGroupsForSSMClient",
+                "Action": ["logs:DescribeLogGroups"],
+                "Resource": ["*"],
+                "Effect": "Allow",
+            }
+        )
+        policy.PolicyDocument["Statement"].append(
+            {
+                "Action": [
+                    "logs:CreateLogStream",
+                    "logs:DescribeLogStreams",
+                    "logs:PutLogEvents",
+                ],
+                "Resource": [arn_value],
+                "Effect": "Allow",
+            }
+        )
+
+
+def set_ecs_cluster_logging_access(settings, policy, role_stack):
+    set_ecs_cluster_logging_kms_access(settings, policy, role_stack)
+    set_ecs_cluster_logging_cw_access(settings, policy, role_stack)
+    set_ecs_cluster_logging_s3_access(settings, policy, role_stack)
+
+
 class ComposeFamily(object):
     """
     Class to group services logically to create the final ECS Service
@@ -1311,10 +1418,12 @@ class ComposeFamily(object):
         self.task_ephemeral_storage = 0
         self.exec_role = None
         self.task_role = None
+        self.enable_execute_command = False
         self.scalable_target = None
         self.ecs_service = None
         self.launch_type = self.default_launch_type
         self.ecs_capacity_providers = []
+        self.target_groups = []
         self.set_compute_platform()
         self.task_logging_options = {}
         self.stack_parameters = {}
@@ -1376,7 +1485,9 @@ class ComposeFamily(object):
                 f"{self.name} - The compute platform is already overridden to {self.launch_type}"
             )
             if self.stack:
-                self.stack.Parameters.update({ecs_params.LAUNCH_TYPE: self.launch_type})
+                self.stack.Parameters.update(
+                    {ecs_params.LAUNCH_TYPE.title: self.launch_type}
+                )
             for service in self.services:
                 setattr(service, "compute_platform", self.launch_type)
         elif not all(
@@ -1392,9 +1503,84 @@ class ComposeFamily(object):
                     self.launch_type = platform
                     if self.stack:
                         self.stack.Parameters.update(
-                            {ecs_params.LAUNCH_TYPE: self.launch_type}
+                            {ecs_params.LAUNCH_TYPE.title: self.launch_type}
                         )
 
+    def set_enable_execute_command(self):
+        for svc in self.services:
+            if svc.is_aws_sidecar:
+                continue
+            if svc.x_ecs and keyisset("EnableExecuteCommand", svc.x_ecs):
+                self.enable_execute_command = True
+        if (
+            self.enable_execute_command
+            and self.task_definition
+            and self.task_definition.ContainerDefinitions
+        ):
+            for container in self.task_definition.ContainerDefinitions:
+                if hasattr(container, "LinuxParameters"):
+                    params = getattr(container, "LinuxParameters")
+                    setattr(params, "InitProcessEnabled", True)
+                else:
+                    setattr(
+                        container,
+                        "LinuxParameters",
+                        LinuxParameters(InitProcessEnabled=True),
+                    )
+
+    def apply_ecs_execute_command_permissions(self, settings):
+        """
+        Method to set the IAM Policies in place to allow ECS Execute SSM and Logging
+
+        :param settings:
+        :return:
+        """
+        policy_title = "EnableEcsExecuteCommand"
+        role_stack = self.task_role.stack
+        task_role = Ref(self.task_role.cfn_resource)
+        if policy_title not in role_stack.stack_template.resources:
+            policy = role_stack.stack_template.add_resource(
+                PolicyType(
+                    policy_title,
+                    PolicyName="EnableExecuteCommand",
+                    PolicyDocument={
+                        "Version": "2012-10-17",
+                        "Statement": [
+                            {
+                                "Effect": "Allow",
+                                "Action": [
+                                    "ssmmessages:CreateControlChannel",
+                                    "ssmmessages:CreateDataChannel",
+                                    "ssmmessages:OpenControlChannel",
+                                    "ssmmessages:OpenDataChannel",
+                                ],
+                                "Resource": "*",
+                            }
+                        ],
+                    },
+                    Roles=[task_role],
+                )
+            )
+            set_ecs_cluster_logging_access(settings, policy, role_stack)
+        elif policy_title in role_stack.stack_template.resources:
+            policy = role_stack.stack_template.resources[policy_title]
+            if hasattr(policy, "Roles"):
+                roles = getattr(policy, "Roles")
+                if roles:
+                    for role in roles:
+                        if (
+                            isinstance(role, Ref)
+                            and role.data["Ref"] != task_role.data["Ref"]
+                        ):
+                            roles.append(task_role)
+            else:
+                setattr(policy, "Roles", [task_role])
+        setattr(
+            self.ecs_service.ecs_service,
+            "EnableExecuteCommand",
+            self.enable_execute_command,
+        )
+
     def set_xray(self):
         """
         Automatically adds the xray-daemon sidecar to the task definition.
@@ -1595,8 +1781,9 @@ class ComposeFamily(object):
                     RetentionInDays=expiry,
                 )
             )
-            policy = Policy(
-                PolicyName=Sub(f"CloudWatchAccessFor${{{ecs_params.SERVICE_NAME_T}}}"),
+            policy = PolicyType(
+                f"CloudWatchAccessFor{self.logical_name}{log_group_title}",
+                PolicyName=f"CloudWatchAccessFor{self.logical_name}{log_group_title}",
                 PolicyDocument={
                     "Version": "2012-10-17",
                     "Statement": [
@@ -1611,11 +1798,10 @@ class ComposeFamily(object):
                         }
                     ],
                 },
+                Roles=[Ref(self.exec_role.name["ImportParameter"])],
             )
-            try:
-                self.exec_role.Policies.append(policy)
-            except AttributeError:
-                setattr(self.exec_role, "Policies", [policy])
+            if self.template and policy.title not in self.template.resources:
+                self.template.add_resource(policy)
             service.logging.Options.update({"awslogs-group": Ref(log_group)})
         else:
             LOG.debug("LOG Group and policy already exist")
@@ -1636,11 +1822,9 @@ class ComposeFamily(object):
                 LOG.warning(
                     "When defining awslogs-region, Compose-X does not create the CW Log Group"
                 )
-                self.exec_role.Policies.append(
+                self.exec_role.cfn_resource.Policies.append(
                     Policy(
-                        PolicyName=Sub(
-                            f"CloudWatchAccessFor${{{ecs_params.SERVICE_NAME_T}}}"
-                        ),
+                        PolicyName=f"CloudWatchAccessFor{self.logical_name}",
                         PolicyDocument={
                             "Version": "2012-10-17",
                             "Statement": [
@@ -1737,15 +1921,15 @@ class ComposeFamily(object):
         self.set_secrets_access()
 
     def handle_permission_boundary(self, prop_key):
-        if keyisset("PermissionsBoundary", self.iam) and self.template:
-            if EXEC_ROLE_T in self.template.resources:
-                add_role_boundaries(
-                    self.template.resources[EXEC_ROLE_T], self.iam[prop_key]
-                )
-            if TASK_ROLE_T in self.template.resources:
-                add_role_boundaries(
-                    self.template.resources[TASK_ROLE_T], self.iam[prop_key]
-                )
+        """
+        Function to add the permissions boundary to IAM roles
+
+        :param str prop_key:
+        :return:
+        """
+        if keyisset(prop_key, self.iam):
+            add_role_boundaries(self.exec_role.cfn_resource, self.iam[prop_key])
+            add_role_boundaries(self.task_role.cfn_resource, self.iam[prop_key])
 
     def assign_iam_policies(self, role, prop):
         """
@@ -1790,9 +1974,10 @@ class ComposeFamily(object):
         """
         if role_name is None:
             role_name = TASK_ROLE_T
-        if not self.template or role_name not in self.template.resources:
-            return
-        role = self.template.resources[role_name]
+        if role_name == EXEC_ROLE_T:
+            role = self.exec_role.cfn_resource
+        else:
+            role = self.task_role.cfn_resource
         props = [
             (
                 "ManagedPolicyArns",
@@ -1814,21 +1999,18 @@ class ComposeFamily(object):
         """
         Method to handle secrets permissions access
         """
-        if (
-            self.template
-            and EXEC_ROLE_T in self.template.resources
-            and TASK_ROLE_T in self.template.resources
-        ):
-            secrets = []
-            for service in self.services:
-                for secret in service.secrets:
-                    secrets.append(secret)
-            if secrets:
-                assign_secrets_to_roles(
-                    secrets,
-                    self.template.resources[EXEC_ROLE_T],
-                    self.template.resources[TASK_ROLE_T],
-                )
+        if not self.exec_role or not self.task_role:
+            return
+        secrets = []
+        for service in self.services:
+            for secret in service.secrets:
+                secrets.append(secret)
+        if secrets:
+            assign_secrets_to_roles(
+                secrets,
+                self.exec_role.cfn_resource,
+                self.task_role.cfn_resource,
+            )
 
     def set_task_compute_parameter(self):
         """
@@ -1884,8 +2066,8 @@ class ComposeFamily(object):
             if 0 < self.task_ephemeral_storage >= 21
             else Ref(AWS_NO_VALUE),
             Family=Ref(ecs_params.SERVICE_NAME),
-            TaskRoleArn=GetAtt(TASK_ROLE_T, "Arn"),
-            ExecutionRoleArn=GetAtt(EXEC_ROLE_T, "Arn"),
+            TaskRoleArn=Ref(self.task_role.arn["ImportParameter"]),
+            ExecutionRoleArn=Ref(self.exec_role.arn["ImportParameter"]),
             ContainerDefinitions=[s.container_definition for s in self.services],
             RequiresCompatibilities=["EC2", "FARGATE"],
             Tags=Tags(
@@ -1918,8 +2100,6 @@ class ComposeFamily(object):
             logging_def.Options.update(self.task_logging_options)
 
     def init_task_definition(self):
-        if self.template:
-            add_service_roles(self)
         self.set_task_compute_parameter()
         self.set_task_definition()
         self.refresh_container_logging_definition()
@@ -1985,13 +2165,14 @@ class ComposeFamily(object):
                 setattr(service.container_definition, "EnvironmentFiles", env_files)
             else:
                 service.container_definition.EnvironmentFiles += env_files
-            if "S3EnvFilesAccess" not in [
-                policy.PolicyName
-                for policy in self.exec_role.Policies
-                if isinstance(policy.PolicyName, str)
-            ]:
-                self.exec_role.Policies.append(
-                    Policy(
+            if (
+                env_files
+                and self.template
+                and "S3EnvFilesAccess" not in self.template.resources
+            ):
+                self.template.add_resource(
+                    PolicyType(
+                        "S3EnvFilesAccess",
                         PolicyName="S3EnvFilesAccess",
                         PolicyDocument={
                             "Version": "2012-10-17",
@@ -2005,6 +2186,10 @@ class ComposeFamily(object):
                                 }
                             ],
                         },
+                        Roles=[
+                            Ref(self.exec_role.name["ImportParameter"]),
+                            Ref(self.task_role.name["ImportParameter"]),
+                        ],
                     )
                 )
 
@@ -2035,22 +2220,24 @@ class ComposeFamily(object):
                 "RepositoryCredentials",
                 RepositoryCredentials(CredentialsParameter=secret_arn),
             )
-            self.exec_role.Policies.append(
-                Policy(
-                    PolicyName="AccessToRepoCredentialsSecret",
-                    PolicyDocument={
-                        "Version": "2012-10-17",
-                        "Statement": [
-                            {
-                                "Effect": "Allow",
-                                "Action": ["secretsmanager:GetSecretValue"],
-                                "Sid": "AccessToRepoCredentialsSecret",
-                                "Resource": [secret_arn],
-                            }
-                        ],
-                    },
-                )
+            policy = PolicyType(
+                "AccessToRepoCredentialsSecret",
+                PolicyName="AccessToRepoCredentialsSecret",
+                PolicyDocument={
+                    "Version": "2012-10-17",
+                    "Statement": [
+                        {
+                            "Effect": "Allow",
+                            "Action": ["secretsmanager:GetSecretValue"],
+                            "Sid": "AccessToRepoCredentialsSecret",
+                            "Resource": [secret_arn],
+                        }
+                    ],
+                },
+                Roles=[Ref(self.exec_role.name["ImportParameter"])],
             )
+            if self.template and policy.title not in self.template.resources:
+                self.template.add_resource(policy)
 
     def set_services_mount_points(self):
         """
@@ -2290,13 +2477,13 @@ class ComposeFamily(object):
         Function to perform a final validation of compute before rendering.
         :param ecs_composex.common.settings.ComposeXSettings settings:
         """
-        if settings.ecs_cluster_platform_override:
-            self.launch_type = settings.ecs_cluster_platform_override
+        if settings.ecs_cluster.platform_override:
+            self.launch_type = settings.ecs_cluster.platform_override
             if hasattr(
                 self.service_definition, "CapacityProviderStrategy"
             ) and isinstance(self.service_definition.CapacityProviderStrategy, list):
                 LOG.warning(
-                    f"{self.name} - Due to Launch Type override to {settings.ecs_cluster_platform_override}"
+                    f"{self.name} - Due to Launch Type override to {settings.ecs_cluster.platform_override}"
                     ", ignoring CapacityProviders"
                     f"{[cap.CapacityProvider for cap in self.service_definition.CapacityProviderStrategy]}"
                 )
@@ -2311,9 +2498,27 @@ class ComposeFamily(object):
                     )
         else:
             self.merge_capacity_providers()
-            self.validate_capacity_providers(settings.ecs_cluster_providers)
+            self.validate_capacity_providers(settings.ecs_cluster.capacity_providers)
             if self.stack:
                 LOG.info(
                     f"{self.name} - Updated {ecs_params.LAUNCH_TYPE.title} to"
                     f" {self.stack.Parameters[ecs_params.LAUNCH_TYPE.title]}"
                 )
+
+    def wait_for_all_policies(self):
+        """
+        Function to ensure the Service does not get created/updated before all policies were set completely
+        """
+        policies = [
+            p.title
+            for p in self.template.resources.values()
+            if isinstance(p, PolicyType)
+        ]
+        if hasattr(self.ecs_service.ecs_service, "DependsOn"):
+            depends_on = getattr(self.ecs_service, "DependsOn")
+            for policy in policies:
+                if policy not in depends_on:
+                    depends_on.append(policy)
+        else:
+            setattr(self.ecs_service.ecs_service, "DependsOn", policies)
+        LOG.debug(self.ecs_service.ecs_service.DependsOn)
