@@ -100,6 +100,9 @@ class EcsCluster(object):
         if self.definition and self.use:
             self.mappings = {CLUSTER_NAME.title: {"Name": self.use}}
             root_stack.stack_template.add_mapping(self.mappings_key, self.mappings)
+            self.cluster_identifier = FindInMap(
+                self.mappings_key, CLUSTER_NAME.title, "Name"
+            )
         elif self.lookup:
             self.lookup_cluster(session)
             root_stack.stack_template.add_mapping(self.mappings_key, self.mappings)
