@@ -1,15 +1,17 @@
-﻿#  -*- coding: utf-8 -*-
+#  -*- coding: utf-8 -*-
 # SPDX-License-Identifier: MPL-2.0
 # Copyright 2020-2021 John Mille <john@compose-x.io>
 
 from os import path
 
+from ecs_composex.common import NONALPHANUM
 from ecs_composex.common.cfn_params import Parameter
 from ecs_composex.common.ecs_composex import X_KEY
 from ecs_composex.vpc.vpc_params import SG_ID_TYPE
 
 MOD_KEY = path.basename(path.dirname(path.abspath(__file__)))
 RES_KEY = f"{X_KEY}{MOD_KEY}"
+MAPPINGS_KEY = NONALPHANUM.sub("", MOD_KEY)
 
 LB_ID_T = "elbv2Id"
 LB_ARN_T = "elbv2Arn"
@@ -47,3 +49,12 @@ TGT_GROUP_NAME_T = "TargetGroupName"
 TGT_GROUP_NAME = Parameter(
     TGT_GROUP_NAME_T, return_value="TargetGroupName", Type="String"
 )
+
+TGT_FULL_NAME_T = "TargetGroupFullName"
+TGT_FULL_NAME = Parameter(
+    TGT_FULL_NAME_T, return_value="TargetGroupFullName", Type="String"
+)
+
+TGT_LB_ARNS = Parameter(
+    "LoadBalancerArns", return_value="LoadBalancerArns", Type="CommaDelimitedList"
+)
