@@ -1,4 +1,4 @@
-﻿#  -*- coding: utf-8 -*-
+#  -*- coding: utf-8 -*-
 # SPDX-License-Identifier: MPL-2.0
 # Copyright 2020-2021 John Mille <john@compose-x.io>
 
@@ -10,7 +10,7 @@ from os import path
 
 import yaml
 from compose_x_common.compose_x_common import keyisset
-from troposphere import AWS_ACCOUNT_ID, AWS_PARTITION, AWS_REGION, Ref, Sub
+from troposphere import AWS_ACCOUNT_ID, AWS_PARTITION, AWS_REGION, Base64, Ref, Sub
 from troposphere.ssm import Parameter as CfnSsmParameter
 from yaml import Loader
 
@@ -103,6 +103,8 @@ def import_value_from_file(resource):
     file_path = path.abspath(resource.parameters["FromFile"])
     with open(file_path, "r") as file_fd:
         value = file_fd.read()
+    if keyisset("EncodeToBase64", resource.parameters):
+        return Base64(value)
     if keyisset("ValidateJson", resource.parameters):
         return handle_json_validation(resource, value, file_path)
     elif keyisset("ValidateYaml", resource.parameters):
@@ -153,14 +155,21 @@ class SsmParameter(XResource):
     policies_scaffolds = get_access_types()
 
     def init_outputs(self):
+        spacer = ""
+        if (
+            self.properties
+            and keyisset("Name", self.properties)
+            and not self.properties["Name"].startswith(r"/")
+        ):
+            spacer = "/"
         self.output_properties = {
             SSM_PARAM_NAME: (self.logical_name, self.cfn_resource, Ref, None),
             SSM_PARAM_ARN: (
                 self.logical_name,
                 self.cfn_resource,
                 Sub,
-                f"arn:{{{AWS_PARTITION}}}:ssm:{{{AWS_REGION}}}:{{{AWS_ACCOUNT_ID}}}:parameter:"
-                f"{self.logical_name.title()}",
+                f"arn:${{{AWS_PARTITION}}}:ssm:${{{AWS_REGION}}}:${{{AWS_ACCOUNT_ID}}}:parameter{spacer}"
+                f"${{{self.cfn_resource.title}}}",
             ),
         }
 
